#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

__copyright__ = '2020, D1FFER'
__docformat__ = 'restructuredtext en'

import time
from threading import Thread

from calibre.gui2.actions import InterfaceAction
from calibre.utils.ipc.job import ParallelJob

if False:
    get_icons = get_resources = None
    _ = None


class MesLivresPluginUI(InterfaceAction):
    name = 'Mes Livres'

    action_spec = ('MesLivres', 'images/icon.png',
                   'Envoyer ma collection sur meslivres.cloud', 'Ctrl+Shift+F3')

    dont_add_to = frozenset(['context-menu',
                               'context-menu-device', 'toolbar-child', 'menubar', 'menubar-device',
                               'context-menu-cover-browser', 'context-menu-split'])

    def __init__(self, parent, site_customization):
        InterfaceAction.__init__(self, parent, site_customization)
        self.webViewWorker = None
        self.shutdown_listener_thread = None

    def genesis(self):
        icon = get_icons('images/icon.png')
        self.qaction.setIcon(icon)
        self.qaction.triggered.connect(self.launch_web_app)

        self.shutdown_listener_thread = WebViewShutdownListener(self)
        self.shutdown_listener_thread.start()

    def shutting_down(self):
        if self.webViewWorker is not None:
            self.webViewWorker.kill()
            self.webViewWorker = None
        if self.shutdown_listener_thread is not None:
            self.shutdown_listener_thread.shutdown()

    def launch_web_app(self):
        if self.webViewWorker:
            return
        db = self.gui.current_db.new_api
        ids = db.all_book_ids()
        paths = []
        for i, id in enumerate(ids):
            formats = db.formats(id)
            for format in formats:
                path = db.format_abspath(id, format)
                paths.append(path)

        # Launch a separate process to view the URL in WebEngine
        self.webViewJob = ParallelJob('webengine-dialog', '', lambda x: x,
                                      args=list(), kwargs={'module': 'calibre_plugins.cloud_meslivres_sync.webview',
                                                           'selected_files': paths})
        server = self.gui.job_manager.server

        self.webViewWorker = server.launch_worker(gui=True, redirect_output=False,
                                                  job_name=getattr(self.webViewJob, 'name', None))
        self.webViewWorker.start_job(self.webViewJob)

        # the lines above are the same as: (example from https://manual.calibre-ebook.com/creating_plugins.html)
        # self.gui.job_manager.launch_gui_app('webengine-dialog',
        #       kwargs={'module':'calibre_plugins.cloud_meslivres_sync.webview', 'selected_files':paths})
        # but keeps a reference to the ConnectedWorker to be able to shut it down when closing Calibre


class WebViewShutdownListener(Thread):
    def __init__(self, ui):
        Thread.__init__(self)
        self.ui = ui
        self._shutdown = False

    def run(self):
        while True:
            if self.ui.webViewWorker is not None and self.ui.webViewWorker.is_alive is False:
                self.ui.webViewWorker = None
            time.sleep(0.5)
            if self._shutdown:
                break

    def shutdown(self):
        self._shutdown = True
