#!/usr/bin/env python
# vim:fileencoding=utf-8
import os
import re
import webbrowser
from pathlib import Path

from PyQt5.Qt import QUrl
from PyQt5.QtCore import QObject, QVariant, pyqtSlot, QFileInfo
from PyQt5.QtWebChannel import QWebChannel
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
from PyQt5.QtWidgets import QFileDialog, QDesktopWidget

from calibre.gui2 import Application
from calibre_plugins.cloud_meslivres_sync.env import web_app_url


class CollectionPage(QWebEnginePage):
    def __init__(self, selected_files):
        super(CollectionPage, self).__init__(None)
        self.selected_files = selected_files
        self.profile().downloadRequested.connect(self.download_file)

    # Called when the file selection dialog is opened in the Web View
    # Returns a list of paths to selected files
    def chooseFiles(self, QWebEnginePage_FileSelectionMode, Iterable, p_str=None, *args, **kwargs):
        return self.selected_files

    def download_file(self, download):
        old_path = download.url().path()
        # the url is a data URL that contains a filename parameter
        match = re.search("filename=(.*),", old_path)
        if match:
            old_path = match.group(1)
        else:
            old_path = 'upload_report.txt'

        suffix = QFileInfo(old_path).suffix()
        path, _ = QFileDialog.getSaveFileName(
            self.view(), "Save File", str(os.path.join(Path.home(), old_path)), "*." + suffix
        )
        if path:
            download.setPath(path)
            download.accept()


class CallHandler(QObject):
    def __init__(self, page):
        super(QObject, self).__init__()
        self.page = page

    @pyqtSlot(result=QVariant)
    def init(self):
        print('Call received from web view')
        return QVariant(None)

    @pyqtSlot()
    def closeWindow(self):
        self.page.view().close()

    @pyqtSlot(result=QVariant, )
    def init(self):
        print('Call received from web view')
        return QVariant(None)

    @pyqtSlot(str)
    def openWindowInDefaultBrowser(self, url):
        print('openWindowInDefaultBrowser({})'.format(url))
        webbrowser.open(url, 1)

    @pyqtSlot(result=QVariant)
    def getSelectedFilesPath(self):
        return QVariant(self.page.selected_files)

def main(selected_files):
    print(selected_files)
    # This function is run in a separate process and can do anything it likes,
    # including use QWebEngine. Here it simply opens the passed in URL
    # in a QWebEngineView
    app = Application([])
    w = QWebEngineView()
    channel = QWebChannel()
    page = CollectionPage(selected_files)
    handler = CallHandler(page)
    channel.registerObject('handler', handler)
    page.setWebChannel(channel)
    w.setPage(page)
    view = page.view()
    width = 550
    height = 550
    view.resize(width, height)
    view.setWindowTitle('MesLivres Calibre')
    center_point = QDesktopWidget().availableGeometry().center()
    view.move(center_point.x() - (width / 2), center_point.y() - (height / 2))
    w.load(QUrl(web_app_url() + '/collection'))
    w.show()
    w.raise_()
    app.exec_()
    return w
